<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;


class PmIntegration
{
    const CLIENT_ID = 'cd7e85178153by60953518945f9hakt19a50a4';
    const CLIENT_SECRET = 'lib5449996c21f780fe30badd6521931a5sac9';

    /** @var EntityManager $entityManager */
    protected $entityManager;

    protected $pmUrl;

    /**
     * PmIntegration constructor.
     * @param EntityManager $entityManager
     * @param $pmUrl
     */
    public function __construct(EntityManager $entityManager, $pmUrl)
    {
        $this->entityManager = $entityManager;
        $this->pmUrl = $pmUrl;
    }

    /**
     * @param $conditions
     * @return array
     * @throws \Exception
     */
    public function findTaskResource($conditions)
    {
        $conditions = $this->conditionsConvertByColumnNameFromName($conditions);
        $data = $this->getDefaultFilterData($conditions);

        $resources = InstanceProvider::getLibUtilities()->callWebService(null, '/api/taskResource', $data, 'taskResource',
            self::CLIENT_SECRET, $this->pmUrl, false);
        return $resources;
    }

    /**
     * @param $conditions
     * @return array
     * @throws \Exception
     */
    public function findTaskParent($conditions)
    {
        $conditions = $this->conditionsConvertByColumnNameFromName($conditions);
        $data = $this->getDefaultFilterData($conditions);
        $resources = InstanceProvider::getLibUtilities()->callWebService(null, '/api/taskParent', $data, 'taskParent',
            self::CLIENT_SECRET, $this->pmUrl, false);
        return $resources;
    }

    /**
     * @param $conditions
     * @return array
     * @throws \Exception
     */
    public function sponsorWithCrud($conditions)
    {
        $conditions = $this->conditionsConvertByColumnNameFromName($conditions);
        $data = $this->getDefaultFilterData($conditions);
        $resources = InstanceProvider::getLibUtilities()->callWebService(null, '/sponsorWithCrud', $data, 'sponsorWithCrud',
            self::CLIENT_SECRET, $this->pmUrl, false);
        return $resources;
    }

    /**
     * @param $row
     * @return array
     * @throws \Exception
     */
    public function saveMultipleProjectTask($row)
    {
        $row['ORGANIZATION_ID'] = InstanceProvider::getDevOpsEnvironment()->getCurrentOrganizationId();
        $row['RESOURCE_ID'] = $this->getCurrentResource();
        $data = array(
            "requestType" => "DataSet",
            "dataset" => array(
                $row
            )
        );
        $ids = InstanceProvider::getLibUtilities()->callWebService(null, '/api/saveMultipleProjectTask/save', $data, 'ID',
            self::CLIENT_SECRET, $this->pmUrl, false);
        return $ids;
    }

    /**
     * @param $row
     * @return array
     * @throws \Exception
     */
    public function commentsIssuesSubRelease($row)
    {
        $row['ORGANIZATION_ID'] = InstanceProvider::getDevOpsEnvironment()->getCurrentOrganizationId();
        $row['RESOURCE_ID'] = $this->getCurrentResource();

        $data = array(
            "requestType" => "DataSet",
            "dataset" => array(
                $row
            )
        );
        $errorLog = InstanceProvider::getLibUtilities()->callWebService(null, '/api/commentsIssuesSubRelease/save', $data, 'errorLog',
            self::CLIENT_SECRET, $this->pmUrl, false);
        return $errorLog;
    }

    public function commentsIssuesRelease($row)
    {
        $row['ORGANIZATION_ID'] = InstanceProvider::getDevOpsEnvironment()->getCurrentOrganizationId();
        $row['RESOURCE_ID'] = $this->getCurrentResource();
        $data = array(
            "requestType" => "DataSet",
            "dataset" => array(
                $row
            )
        );
        $errorLog = InstanceProvider::getLibUtilities()->callWebService(null, '/api/commentsIssuesRelease/save', $data, 'errorLog',
            self::CLIENT_SECRET, $this->pmUrl, false);
        return $errorLog;
    }

    public function saveNewProjectTask($row)
    {
        $row['ORGANIZATION_ID'] = InstanceProvider::getDevOpsEnvironment()->getCurrentOrganizationId();
        $row['RESOURCE_ID'] = $this->getCurrentResource();
        $data = array(
            "requestType" => "DataSet",
            "dataset" => array(
                $row
            )
        );
        $errorLog = InstanceProvider::getLibUtilities()->callWebService(null, '/api/saveNewProjectTask/save', $data, 'ID',
            self::CLIENT_SECRET, $this->pmUrl, false);
        return $errorLog;
    }

    public function getTokenInPm()
    {
        $param = array(
            'CLIENT_ID' => self::CLIENT_ID,
            'CLIENT_SECRET' => self::CLIENT_SECRET
        );
        $token = InstanceProvider::getLibUtilities()->callWebService(null, '/lib_requestToken', $param, 'token',
            self::CLIENT_SECRET, $this->pmUrl, false);
        return $token[0];
    }

    private function getDefaultFilterData($filter)
    {
        $filterData = array(
            'requestType' => 'FilterData',
            'filter' => $filter
        );
        return $filterData;
    }

    public function conditionsConvertByColumnNameFromName($conditions)
    {
        foreach ($conditions as &$c) {
            $c['name'] = $c['columnName'];
            unset($c['columnName']);
        }
        return $conditions;
    }

    public function getCurrentResource()
    {
        $cdoperador = InstanceProvider::getDevOpsEnvironment()->getCurrentCdoperador();
        $resource = InstanceProvider::getRepository()->getResource()->findOneBy(array('externalCode' => $cdoperador));
        return $resource->getId();
    }

}